/*
 * Klogwatch, Netfilter log monitor
 *
 * Copyright (C) 2004 Nick Battle <nick.battle@freeuk.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <qapplication.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qwidgetlist.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qiconset.h>
#include <qtextbrowser.h>
#include <unistd.h>
#include <errno.h>
#include "klogwatch.h"
#include "klogsetup.h"
#include "klogicons.h"
#include "qthelptext.h"
#include "debug.h"

KLogWatch::KLogWatch(const char *logfile) :
    QMainWindow(0, 0, WType_TopLevel | WDestructiveClose | WStyle_ContextHelp)
{
    file = logfile;
    kloglist = new KLogList(file, this);
    setCentralWidget(kloglist);

    connect(kloglist, SIGNAL(sigHide()), this, SLOT(slotHide()));
    connect(kloglist, SIGNAL(sigShow(bool)), this, SLOT(slotShow(bool)));
    connect(kloglist, SIGNAL(sigExit()), this, SLOT(slotExit()));

    QString caption("Netfilter Log Events from ");
    setCaption(caption + file);
    menubar = menuBar();

    QPopupMenu *filePopup = new QPopupMenu();
    QPopupMenu *helpPopup = new QPopupMenu();

    filePopup->insertItem(QIconSet(*klog_filesave),
                "&Save", this, SLOT(slotSave()), CTRL+Key_S);

    filePopup->insertItem(QIconSet(*klog_configure),
                "S&etup", this, SLOT(slotSetup()), CTRL+Key_C);

    filePopup->insertItem(QIconSet(*klog_printer),
                "&Print", this, SLOT(slotPrint()), CTRL+Key_P);

    filePopup->insertItem(QIconSet(*klog_exit),
                 "E&xit", this, SLOT(slotExit()), Key_Escape);
    
    helpPopup->insertItem(QIconSet(*klog_help),
                 "&Help", this, SLOT(slotHelp()), CTRL+Key_H);
    
    helpPopup->insertItem(QIconSet(*klog_app),
                 "&About QLogWatch", this, SLOT(slotAbout()), CTRL+Key_A);
    
    helpPopup->insertItem("A&bout Qt", this, SLOT(slotAboutQt()), CTRL+Key_Q);
    
    menubar->insertItem("&File", filePopup);
    menubar->insertItem("&Help", helpPopup);

    resize(minimumSizeHint());	// force minimal size
}

KLogWatch::~KLogWatch()
{
    debug("Destroy KLogWatch\n");
}

void KLogWatch::slotHide()
{
    debug("Hiding\n");
    kloglist->markOldPackets();
    hide();
}

void KLogWatch::slotShow(bool force)
{
    debug("Showing\n");

    if (force || klogconfig->TRAYALERT == 0)
    {
	debug("Setting active focus\n");
	kloglist->setFocus();
	show();
	raise();
	setActiveWindow();
    }
}

void KLogWatch::slotSetup()
{
    bool repeat = true;
    debug("Setup\n");

    while (repeat)
    {
	KLogSetup *dialog = new KLogSetup(this);
	int choice = dialog->exec();
	delete dialog;

	if (choice == QDialog::Rejected)
	    break;

	int result = QMessageBox::warning(this, "Restart",
	    "You need to restart QLogWatch for your changes to take effect\n"
	    "Do you want to restart the program now?",
	    "Restart Now", "Continue");

        const char *argv0 = qApp->argv()[0];

	if (result == 0)
	{
	    debug("Running: %s\n", argv0);
	    execlp(argv0, argv0, NULL);
	    debug("Exec failed: %s\n", strerror(errno));
	}
	else
	{
	    debug("No\n");
	    repeat = false;
	}
    }
}

void KLogWatch::slotSave()
{
    debug("Saving\n");
    kloglist->thinking();

    QString filename =
	QFileDialog::getSaveFileName(QString::null, QString::null, this);

    if (!filename.isNull())
    {
	QFile file(filename);

	if (file.open(IO_WriteOnly))
	{
	    QTextStream s(&file);
	    kloglist->saveTo(s);
	    file.close();
	}
	else
	{
	    QMessageBox::critical(this, "Save",
	    	QString("Cannot open file '%1'").arg(filename));
	}
    }
}

void KLogWatch::slotPrint()
{
    debug("Print\n");
    kloglist->thinking();
    QPrinter print;

    if (print.setup(this))
    {
        kloglist->printTo(print);
    }
}

void KLogWatch::slotAbout(void)
{
    QMessageBox::about(this, "About QLogWatch",
#include "klogabout.h"
    );
}

void KLogWatch::slotHelp(void)
{
    QtHelpText help(this, "help", true);

    QString text(
#include "klogwatch.txt"
    );

    QFont f;	// App default
    help.textWindow->setFont(f);
    help.textWindow->setText(text);
    help.exec();
}

void KLogWatch::slotAboutQt(void)
{
    QMessageBox::aboutQt(this);
}

void KLogWatch::slotExit()
{
    debug("Exiting\n");
    qApp->exit(0);
}

void KLogWatch::closeEvent(QCloseEvent *e)
{
    e->ignore();

    // if the tray is visible, we don't want to close the program,
    // just hide the log window

    if (klogconfig->SYSTRAY)
    {
	kloglist->running();  // are we "running" or "thinking" here?
	kloglist->markOldPackets();
	hide();
    }
    else
    {
	// otherwise, we'll ask if we really want to close

	if (QMessageBox::warning(this, "Close",
		"Do you want to close QLogWatch?", "Yes", "No") == 0)
	{
	    slotExit();
	}
    }
}

// eof
