/*
 * Klogwatch, Netfilter log monitor
 *
 * Copyright (C) 2004 Nick Battle <nick.battle@freeuk.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <qapplication.h>
#include <qmessagebox.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "klogwatch.h"
#include "klogsetup.h"
#include "klogpid.h"
#include "klogicons.h"
#include "debug.h"


bool debugOn = false;	// See debug.h
KLogConfig *klogconfig = NULL;

int main(int argc, char *argv[])
{
    bool kArg = false, pArg = false, dArg = false, fArg = false, err = false;
    char *fValue = NULL;

    for (int i=1; i<argc; i++)
    {
	if (strcmp(argv[i], "-d") == 0)
	{
	    dArg = true;
	}
	else if (strcmp(argv[i], "-k") == 0)
	{
	    kArg = true;
	}
	else if (strcmp(argv[i], "-p") == 0)
	{
	    pArg = true;
	}
	else if (strcmp(argv[i], "-f") == 0)
	{
	    fArg = true;

	    if (++i < argc)
	    {
		fValue = argv[i];
	    }
	    else
	    {
		err = true;
		break;
	    }
	}
	else
	{
	    err = true;
	    break;
	}
    }

    if (err)
    {
	fprintf(stderr, "Usage: %s [-p|-k|-d|-f <file>]\n", argv[0]);
	exit(1);
    }

    debugOn = dArg;

    if (kArg)
    {
        return KLogWatchKill(SIGTERM);
    }
    else if (pArg)
    {
        return KLogWatchKill(SIGUSR1);
    }
    else
    {
	if (!dArg && daemon(1,0) < 0)	// don't daemonize for debug mode
	{
	    fprintf(stderr, "Can't fork daemon, %s\n", strerror(errno));
	    exit(1);
	}

	QApplication *app = new QApplication(argc, argv);

	klog_setIcons();
        KLogSigInit();
	klogconfig = KLogConfig::getConfig();
        QCString file;
	
	if (fArg)
	{
	    file = QCString(fValue);
	    klogconfig->FILENAME = file;
	}
	else
	{
	    file = klogconfig->FILENAME;
	}

	// Check the file is readable first, else open Setup

	while (true)
	{
	    int fd = open((const char *)file, O_RDONLY);
	    struct stat buf;

	    if ((fd >= 0) && (fstat(fd, &buf) >= 0) && (S_ISREG(buf.st_mode)))
	    {
		debug("Logfile %s OK\n", (const char *)file);
		break;
	    }

            QMessageBox::warning(0, "QLogWatch",
	    	QString("Cannot open Netfilter log file '%1'\n"
		    "File must be readable and not a directory\n"
		    "Please update setup filename or use -f...").arg(file));

	    KLogSetup *dialog = new KLogSetup(NULL);
	    int choice = dialog->exec();
	    delete dialog;

	    if (choice == QDialog::Rejected)
	    {
		debug("Give up\n");
		exit(1);
	    }

	    file = klogconfig->FILENAME;
	}

        QWidget *d = QApplication::desktop();
        int screenh = d->height();
        int screenw = d->width();
        debug("Screen = %dx%d\n", screenw, screenh);
        int rv = 0;

        try
        {
            KLogWatch *main = new KLogWatch(file);

	    // This actions the resizes so that the code that
	    // follows can ask for the width/height and get the
	    // right answers, even though the widget has not 
	    // been shown.

	    QApplication::sendPostedEvents();

            main->move((screenw - main->width())/2,
                       (screenh - main->height())/2);

	    if (dArg)
	    {
		main->show();
	    }

            rv = app->exec();
        }
        catch (QString ex)
        {
            QMessageBox::warning(0, "Error",
	    			QString("%1: %2").arg(file).arg(ex));
            rv = 1;
        }
        catch (...)
        {
            QMessageBox::warning(0, "Error",
	    			QString("Cannot open file '%1'").arg(file));
            rv = 1;
        }

        return rv;
    }

    return 0;
}

// eof
